package de.cliffbreak.varo;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.configuration.file.YamlConfiguration;

import de.cliffbreak.varo.commands.CoordsCommand;
import de.cliffbreak.varo.commands.VaroCommand;
import de.cliffbreak.varo.commands.VaroTabCompleter;
import de.cliffbreak.varo.listeners.BannedActionListener;
import de.cliffbreak.varo.listeners.BannedItemListener;
import de.cliffbreak.varo.listeners.ChatListener;
import de.cliffbreak.varo.listeners.PlayerInteractNPCListener;
import de.cliffbreak.varo.listeners.EntityRegainHealthListener;
import de.cliffbreak.varo.listeners.PlayerClientOptionsChangeListener;
import de.cliffbreak.varo.listeners.PlayerDeathListener;
import de.cliffbreak.varo.listeners.PlayerJoinQuitListener;
import de.cliffbreak.varo.listeners.PlayerLoginListener;
import de.cliffbreak.varo.listeners.PlayerResourcePackStatusListener;
import de.cliffbreak.varo.managers.NPCManager;
import de.cliffbreak.varo.uitls.BanUtils;
import de.cliffbreak.varo.uitls.PlayerCache;
import de.cliffbreak.varo.uitls.VaroUtils;

public class Varo extends JavaPlugin {

    private File configurationFile;
    private FileConfiguration config;
    private NPCManager npcManager;
    private PlayerCache playerCache;
    private BanUtils banUtils;
    private VaroUtils varoUtils;

    private boolean canCreatePortal = false;

    @Override
    public void onEnable() {

        this.getLogger().info("CliffbreakVaro is starting...");

        this.configurationFile = new File("plugins/CliffbreakVaro", "config.yml");
        this.config = YamlConfiguration.loadConfiguration(this.configurationFile);

        this.npcManager = new NPCManager(this);
        this.playerCache = new PlayerCache(this);
        this.banUtils = new BanUtils(this);
        this.varoUtils = new VaroUtils(this);

        // this.config.addDefault("Varo.Start", "TODO: StartDate");
        this.config.addDefault("Varo.Debug", false);
        this.config.addDefault("Varo.Bans", new ArrayList<String>());
        this.config.options().copyDefaults(true);
        this.saveConfig();

        this.getServer().getPluginManager().registerEvents(new PlayerLoginListener(this), this);
        this.getServer().getPluginManager().registerEvents(new PlayerJoinQuitListener(this), this);
        this.getServer().getPluginManager().registerEvents(new PlayerClientOptionsChangeListener(this), this);
        this.getServer().getPluginManager().registerEvents(new PlayerResourcePackStatusListener(), this);
        this.getServer().getPluginManager().registerEvents(new ChatListener(), this);
        this.getServer().getPluginManager().registerEvents(new EntityRegainHealthListener(), this);
        this.getServer().getPluginManager().registerEvents(new PlayerDeathListener(this), this);
        this.getServer().getPluginManager().registerEvents(new BannedItemListener(), this);
        this.getServer().getPluginManager().registerEvents(new BannedActionListener(this), this);
        this.getServer().getPluginManager().registerEvents(new PlayerInteractNPCListener(this), this);

        this.getCommand("varo").setExecutor(new VaroCommand(this));
        this.getCommand("varo").setTabCompleter(new VaroTabCompleter(this));
        this.getCommand("coords").setExecutor(new CoordsCommand(this));

        this.varoUtils.init();
    }

    @Override
    public void onDisable() {
        this.getLogger().info("Stopping CliffbreakVaro!");
        this.playerCache.shutdown();
    }

    @Override
    public FileConfiguration getConfig() {
        return this.config;
    }

    @Override
    public void saveConfig() {
        try {
            this.getLogger().info("Saving Config...");
            this.config.save(this.configurationFile);
        } catch (final IOException e) {
            e.printStackTrace();
        }
    }

    public NPCManager getNPCManager() {
        return npcManager;
    }

    public PlayerCache getPlayerCache() {
        return playerCache;
    }

    public BanUtils getBanUtils() {
        return this.banUtils;
    }

    public TeamManager getTeamManager() {
        return teamManager;
    }

    public boolean canCreatePortal() {
        return this.canCreatePortal;
    }

    public void setCanCreatePortal(final boolean canCreatePortal) {
        this.canCreatePortal = canCreatePortal;
    }
}
